from django.conf import settings
from django.contrib import messages
from django.shortcuts import resolve_url
from django.utils.translation import ugettext_lazy as _
from django.views import generic

from user_management.utils.views import VerifyAccountViewMixin
from .exceptions import AlreadyVerifiedException, InvalidExpiredToken


class VerifyUserEmailView(VerifyAccountViewMixin, generic.RedirectView):
    """
    A view which verifies a user's email address.

    Accessed via a link in an email sent to the user, which contains both a
    uid generated from the user's pk, and a token also generated from the user
    object, for verification.  If everything lines up, it makes the user
    active.

    As a RedirectView, this will return a HTTP 302 to LOGIN_URL on success.
    """
    permanent = False
    already_verified = False
    success_message = _('Your email address was confirmed.')
    already_verified_message = _('Your email is already confirmed.')
    invalid_exception_class = InvalidExpiredToken
    permission_denied_class = AlreadyVerifiedException

    def get_redirect_url(self, *args, **kwargs):
        return resolve_url(settings.LOGIN_URL)

    def dispatch(self, request, *args, **kwargs):
        try:
            self.verify_token(request, *args, **kwargs)
        except self.permission_denied_class:
            self.already_verified = True
            self.success_message = self.already_verified_message
        return super(VerifyUserEmailView, self).dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        if not self.already_verified:
            self.activate_user()
        messages.success(request, self.success_message)
        return super(VerifyUserEmailView, self).get(request, *args, **kwargs)



from django.http import Http404
from django.utils.translation import ugettext_lazy as _


class InvalidExpiredToken(Http404):
    """Exception to confirm an account."""
    message = _('Invalid or expired token.')


class AlreadyVerifiedException(Exception):
    message = _('Email already verified.')



from django.conf.urls import include, url


urlpatterns = [
    url(r'', include('user_management.api.avatar.urls.profile_avatar')),
    url(r'', include('user_management.api.avatar.urls.user_avatar')),
]



from django.test import override_settings

from user_management.models.tests.factories import VerifyEmailUserFactory
from user_management.models.tests.models import VerifyEmailUser
from user_management.models.tests.utils import RequestTestCase
from .. import views


@override_settings(AUTH_USER_MODEL='tests.VerifyEmailUser')
class TestVerifyUserEmailView(RequestTestCase):
    view_class = views.VerifyUserEmailView

    def test_get(self):
        """A user clicks the link in their email and activates their account."""
        user = VerifyEmailUserFactory.create(email_verified=False)
        token = user.generate_validation_token()

        request = self.create_request('get', auth=False)
        view = self.view_class.as_view()
        response = view(request, token=token)
        self.assertEqual(response.status_code, 302)

        self.assertEqual(response.url, '/accounts/login/')

        user = VerifyEmailUser.objects.get(pk=user.pk)

        self.assertTrue(user.email_verified)
        self.assertTrue(user.is_active)

        self.assertEqual(
            self.view_class.success_message,
            str(request._messages.store[0]),
        )

    @override_settings(LOGIN_URL='login')
    def test_get_named_login_url(self):
        """A user clicks the link in their email and activates their account."""
        user = VerifyEmailUserFactory.create(email_verified=False)
        token = user.generate_validation_token()

        request = self.create_request('get', auth=False)
        view = self.view_class.as_view()
        response = view(request, token=token)
        self.assertEqual(response.status_code, 302)

        self.assertEqual(response.url, '/login/')

    def test_get_nonsense_token(self):
        """The view is accessed with a broken token and 404s."""
        token = 'I_am_a_token'

        request = self.create_request('get', auth=False)
        view = self.view_class.as_view()
        with(self.assertRaises(self.view_class.invalid_exception_class)):
            view(request, token=token)

    def test_get_registered_user(self):
        """The view is accessed for an already-verified user then redirect."""
        user = VerifyEmailUserFactory.create(email_verified=True)
        token = user.generate_validation_token()

        request = self.create_request('get', auth=False)
        view = self.view_class.as_view()

        response = view(request, token=token)
        self.assertEqual(response.status_code, 302)

        self.assertEqual(response.url, '/accounts/login/')

        user = VerifyEmailUser.objects.get(pk=user.pk)

        self.assertTrue(user.email_verified)
        self.assertTrue(user.is_active)

        self.assertEqual(
            self.view_class.already_verified_message,
            str(request._messages.store[0]),
        )



from django.conf import settings
from django.contrib.auth import get_user_model
from django.core import signing
from django.utils.translation import ugettext_lazy as _
from rest_framework.exceptions import PermissionDenied

from user_management.api.exceptions import InvalidExpiredToken


class VerifyAccountViewMixin(object):
    """
    Verify a new user's email address.

    Verify a newly created account by checking the `token` in the URL kwargs.
    """
    ok_message = _('Your account has been verified.')
    invalid_exception_class = InvalidExpiredToken
    permission_denied_class = PermissionDenied

    # Default token never expires.
    DEFAULT_VERIFY_ACCOUNT_EXPIRY = None

    def verify_token(self, request, *args, **kwargs):
        """
        Use `token` to allow one-time access to a view.

        Set the user as a class attribute or raise an `InvalidExpiredToken`.

        Token expiry can be set in `settings` with `VERIFY_ACCOUNT_EXPIRY` and is
        set in seconds.
        """
        User = get_user_model()

        try:
            max_age = settings.VERIFY_ACCOUNT_EXPIRY
        except AttributeError:
            max_age = self.DEFAULT_VERIFY_ACCOUNT_EXPIRY

        try:
            email_data = signing.loads(kwargs['token'], max_age=max_age)
        except signing.BadSignature:
            raise self.invalid_exception_class

        email = email_data['email']

        try:
            self.user = User.objects.get_by_natural_key(email)
        except User.DoesNotExist:
            raise self.invalid_exception_class

        if self.user.email_verified:
            raise self.permission_denied_class

    def activate_user(self):
        self.user.email_verified = True
        self.user.is_active = True
        self.user.save()



import mock
from django.contrib.auth import get_user_model
from django.test import override_settings

from user_management.models.tests.factories import VerifyEmailUserFactory
from user_management.models.tests.utils import APIRequestTestCase
from .. import views

User = get_user_model()


@override_settings(AUTH_USER_MODEL='tests.VerifyEmailUser')
class TestVerifyAccountView(APIRequestTestCase):
    view_class = views.VerifyAccountViewMixin

    @classmethod
    def setUpTestData(cls):
        cls.request = mock.MagicMock
        cls.view_instance = cls.view_class()
        cls.user = VerifyEmailUserFactory.create(email_verified=False)
        cls.token = cls.user.generate_validation_token()

    def test_verify_token_allowed(self):
        """Assert a user can verify its own email."""
        self.view_instance.verify_token(self.request, token=self.token)
        self.assertEqual(self.view_instance.user, self.user)

    def test_verify_token_invalid_user(self):
        """Assert a nonexistent user throws an exception."""
        user = VerifyEmailUserFactory.build()
        token = user.generate_validation_token()
        with self.assertRaises(self.view_instance.invalid_exception_class):
            self.view_instance.verify_token(self.request, token=token)

    def test_verify_token_invalid_token(self):
        """Assert forged token return a bad request."""
        token = 'nimporte-nawak'
        with self.assertRaises(self.view_instance.invalid_exception_class):
            self.view_instance.verify_token(self.request, token=token)

    def test_default_expiry_token(self):
        """Assert `DEFAULT_VERIFY_ACCOUNT_EXPIRY` doesn't expire by default."""
        with mock.patch('django.core.signing.loads') as signing_loads:
            signing_loads.return_value = {'email': self.user.email}
            self.view_instance.verify_token(self.request, token=self.token)

        signing_loads.assert_called_once_with(self.token, max_age=None)

    @override_settings(VERIFY_ACCOUNT_EXPIRY=0)
    def test_verify_token_expired_token(self):
        """Assert token expires when VERIFY_ACCOUNT_EXPIRY is set."""
        with self.assertRaises(self.view_instance.invalid_exception_class):
            self.view_instance.verify_token(self.request, token=self.token)

    def test_verify_token_verified_email(self):
        """Assert verified user cannot verify email."""
        user = VerifyEmailUserFactory.create(email_verified=True)
        token = user.generate_validation_token()
        with self.assertRaises(self.view_instance.permission_denied_class):
            self.view_instance.verify_token(self.request, token=token)

    def test_activate_user(self):
        user = VerifyEmailUserFactory.create(email_verified=False, is_active=False)
        self.view_instance.user = user

        self.view_instance.activate_user()
        self.assertTrue(user.email_verified)
        self.assertTrue(user.is_active)
